/*
 * @(#)MPConstraintMatrix.java	1.00 98/12/28
 * 
 * Modification Log:
 *
 */

package mplayout;

import java.util.Vector;


/**
 * Internal class used to manage the matrix of constraints for the layout.
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 28 Dec 1998
 */

final class MPConstraintMatrix
{
    /** type of constraint */
    public static final int     VERT_TYPE=0, HORZ_TYPE=1;
    /** the final cell dimensions are stored here */
    public int                  aiVertSizes[], aiHorzSizes[];

    /** vectors to store constraints for calculation */
    private Vector              aVertConstr[], aHorzConstr[];
    /** size of this matrix */
    private int                 iRows, iCols;
    /** true if calculation has been done false otherwise */
    private boolean             bCalcDone;

    /**
     * Constructor.
     * @param   iCols   the number of columns this matrix will have
     * @param   iRows   the number of rows this matrix will have
     */
	public MPConstraintMatrix(int iCols, int iRows)
	{
        this.iRows = iRows;
        this.iCols = iCols;
        aVertConstr = new Vector[iRows];
        aHorzConstr = new Vector[iCols];
        aiVertSizes = new int[iRows];
        aiHorzSizes = new int[iCols];
	}

    /**
     * Add a new constraint for a cell. A constraint is a formula that defines
     * the value of a cell corresponding to that of another cell.
     * 
     * @param   iType       whether a vertical or horizontal constraint
     * @param   iForCell    the cell number (vertical or horizontal depending on iType) 
     *                      for which this is meant.
     * @param   iStartCell  the cell to which the constraint refers.
     * @param   iIncrement  the increment value from the value of the starting cell denoted by iStartCell
     */
	public void addConstraint(int iType, int iForCell, int iStartCell, int iIncrement)
	{
        if (iForCell < 0)   iForCell = 0;
        switch(iType)
        {
            case VERT_TYPE:
                //System.out.println("Added vert constr to " + iForCell);
                if (null == aVertConstr[iForCell])
                {
                    aVertConstr[iForCell] = new Vector();
                }
                aVertConstr[iForCell].addElement(new MPConstraint(iStartCell, iIncrement));
                break;
            case HORZ_TYPE:
                //System.out.println("Added horz constr to " + iForCell);
                if (null == aHorzConstr[iForCell])
                {
                    aHorzConstr[iForCell] = new Vector();
                }
                aHorzConstr[iForCell].addElement(new MPConstraint(iStartCell, iIncrement));
                break;
        }
	}

    /**
     * Solve the matrix and find out the grid sizes.
     */
    public void solveMatrix()
    {
        int             iIndex, iIndex2, iThisValue;
        Vector          vect;
        MPConstraint    thisConstr;

        if (bCalcDone)
        {
            resetElements();
        }

        System.out.println("Solving Rows...");
        // solve the rows
        for (iIndex=0; iIndex<iRows; iIndex++)
        {
            aiVertSizes[iIndex] = ((iIndex>0) ? aiVertSizes[iIndex-1] : 0) + MPConstraint.DEF_SIZE;
            if (null != (vect = aVertConstr[iIndex]))
            {
                for (iIndex2 = vect.size(); iIndex2 > 0; iIndex2--)
                {
                    thisConstr = (MPConstraint)vect.elementAt(iIndex2-1);
                    iThisValue = thisConstr.iIncrement;
                    if (thisConstr.iStartCell >= 0)
                    {
                        iThisValue += aiVertSizes[thisConstr.iStartCell];
                    }
                    if (iThisValue > aiVertSizes[iIndex])
                    {
                        aiVertSizes[iIndex] = iThisValue;
                    }
                }
            }
            //System.out.print("," + aiVertSizes[iIndex]);
        }
        //System.out.println("");

        System.out.println("Solving Cols...");
        // solve the columns
        for (iIndex=0; iIndex<iCols; iIndex++)
        {
            aiHorzSizes[iIndex] = ((iIndex>0) ? aiHorzSizes[iIndex-1] : 0) + MPConstraint.DEF_SIZE;
            if (null != (vect = aHorzConstr[iIndex]))
            {
                for (iIndex2 = vect.size(); iIndex2 > 0; iIndex2--)
                {
                    thisConstr = (MPConstraint)vect.elementAt(iIndex2-1);
                    iThisValue = thisConstr.iIncrement;
                    if (thisConstr.iStartCell >= 0)
                    {
                        iThisValue += aiHorzSizes[thisConstr.iStartCell];
                    }
                    if (iThisValue > aiHorzSizes[iIndex])
                    {
                        aiHorzSizes[iIndex] = iThisValue;
                    }
                }
            }
            //System.out.print("," + aiHorzSizes[iIndex]);
        }
        //System.out.println("");
        bCalcDone = true;
    }

    /**
     * Clean up the component list so that new components can be added and the
     * calculation can be done again.
     */
    public void reset()
    {
        aVertConstr = new Vector[iRows];
        aHorzConstr = new Vector[iCols];
        aiVertSizes = new int[iRows];
        aiHorzSizes = new int[iCols];
        bCalcDone = false;
    }
    
    
    /**
     * Reset the calculations so that a fresh calculation can be done on the 
     * existing component list.
     * Do not clean up the vectors. Just reset their values.
     */
    public void resetElements()
    {
        int             iIndex, iIndex2;
        Vector          vect;
        MPConstraint    thisConstr;

        for (iIndex=0; iIndex<iRows; iIndex++)
        {
            if (null != (vect = aVertConstr[iIndex]))
            {
                for (iIndex2 = vect.size(); iIndex2 > 0; iIndex2--)
                {
                    thisConstr = (MPConstraint)vect.elementAt(iIndex2-1);
                    thisConstr.reset();
                }
            }
            aiVertSizes[iIndex] = (iIndex>0) ? aiVertSizes[iIndex-1] : 0 + MPConstraint.DEF_SIZE;
        }

        for (iIndex=0; iIndex<iCols; iIndex++)
        {
            if (null != (vect = aHorzConstr[iIndex]))
            {
                for (iIndex2 = vect.size(); iIndex2 > 0; iIndex2--)
                {
                    thisConstr = (MPConstraint)vect.elementAt(iIndex2-1);
                    thisConstr.reset();
                }
            }
            aiHorzSizes[iIndex] = (iIndex>0) ? aiHorzSizes[iIndex-1] : 0 + MPConstraint.DEF_SIZE;
        }
        bCalcDone = false;
    }



    /**
     * Fit the grids to this size. Resizing them in the current proportion.
     */
    public void fitMatrixTo(int iWidth, int iHeight)
    {
        int iIndex, iMax;

        // for vertical sizes
        iMax = aiVertSizes.length;
        for (iIndex=0; iIndex<iMax; iIndex++)
        {
            aiVertSizes[iIndex] = iHeight*aiVertSizes[iIndex]/aiVertSizes[iMax-1];
        }

        // for horizontal sizes
        iMax = aiHorzSizes.length;
        for (iIndex=0; iIndex<iMax; iIndex++)
        {
            aiHorzSizes[iIndex] = iWidth*aiHorzSizes[iIndex]/aiHorzSizes[iMax-1];
        }
    }
}

